@@ -1,10 +1,8 @@
 ##################################################################
-# Copyright 2016 OSGeo Foundation,                               #
-# represented by PyWPS Project Steering Committee,               #
+# Copyright 2018 Open Source Geospatial Foundation and others    #
 # licensed under MIT, Please consult LICENSE.txt for details     #
 ##################################################################
 
-
 import logging
 
 LOGGER = logging.getLogger("PYWPS")
@@ -16,16 +14,22 @@ class Metadata(object):
 
     :param title: Metadata title, human readable string
     :param href: fully qualified URL
+    :param role: fully qualified URL
     :param type_: fully qualified URL
     """
 
-    def __init__(self, title, href=None, type_='simple'):
+    def __init__(self, title, href=None, role=None, type_='simple'):
         self.title = title
         self.href = href
+        self.role = role
         self.type = type_
 
     def __iter__(self):
-        yield '{http://www.w3.org/1999/xlink}title', self.title
+        metadata = {"title": self.title}
+
         if self.href is not None:
-            yield '{http://www.w3.org/1999/xlink}href', self.href
-        yield '{http://www.w3.org/1999/xlink}type', self.type
+            metadata['href'] = self.href
+        if self.role is not None:
+            metadata['role'] = self.role
+        metadata['type'] = self.type
+        yield metadata
