@@ -1,10 +1,8 @@
 ##################################################################
-# Copyright 2016 OSGeo Foundation,                               #
-# represented by PyWPS Project Steering Committee,               #
+# Copyright 2018 Open Source Geospatial Foundation and others    #
 # licensed under MIT, Please consult LICENSE.txt for details     #
 ##################################################################
 
-
 import logging
 import os
 import sys
@@ -13,9 +11,10 @@ import json
 import shutil
 import tempfile
 
-from pywps import WPS, OWS, E, dblog
-from pywps.app.WPSResponse import WPSResponse
-from pywps.app.WPSResponse import STATUS
+from pywps import get_ElementMakerForVersion, E, dblog
+from pywps.response import get_response
+from pywps.response.status import WPS_STATUS
+from pywps.response.execute import ExecuteResponse
 from pywps.app.WPSRequest import WPSRequest
 import pywps.configuration as config
 from pywps._compat import PY2
@@ -32,7 +31,10 @@ class Process(object):
                     request. It should accept a single
                     :class:`pywps.app.WPSRequest` argument and return a
                     :class:`pywps.app.WPSResponse` object.
-    :param identifier: Name of this process.
+    :param string identifier: Name of this process.
+    :param string title: Human readable title of process.
+    :param string abstract: Brief narrative description of the process.
+    :param list keywords: Keywords that characterize a process.
     :param inputs: List of inputs accepted by this process. They
                    should be :class:`~LiteralInput` and :class:`~ComplexInput`
                    and :class:`~BoundingBoxInput`
@@ -45,12 +47,13 @@ class Process(object):
                      should be :class:`pywps.app.Common.Metadata` objects.
     """
 
-    def __init__(self, handler, identifier, title, abstract='', profile=[], metadata=[], inputs=[],
+    def __init__(self, handler, identifier, title, abstract='', keywords=[], profile=[], metadata=[], inputs=[],
                  outputs=[], version='None', store_supported=False, status_supported=False, grass_location=None):
         self.identifier = identifier
         self.handler = handler
         self.title = title
         self.abstract = abstract
+        self.keywords = keywords
         self.metadata = metadata
         self.profile = profile
         self.version = version
@@ -62,6 +65,7 @@ class Process(object):
         self.workdir = None
         self._grass_mapset = None
         self.grass_location = grass_location
+        self.service = None
 
         if store_supported:
             self.store_supported = 'true'
@@ -73,60 +77,28 @@ class Process(object):
         else:
             self.status_supported = 'false'
 
-    def capabilities_xml(self):
-        doc = WPS.Process(
-            OWS.Identifier(self.identifier),
-            OWS.Title(self.title)
-        )
-        if self.abstract:
-            doc.append(OWS.Abstract(self.abstract))
-        for m in self.metadata:
-            doc.append(OWS.Metadata(dict(m)))
-        if self.profile:
-            doc.append(OWS.Profile(self.profile))
-        if self.version != 'None':
-            doc.attrib['{http://www.opengis.net/wps/1.0.0}processVersion'] = self.version
-        else:
-            doc.attrib['{http://www.opengis.net/wps/1.0.0}processVersion'] = 'undefined'
+    @property
+    def json(self):
 
-        return doc
-
-    def describe_xml(self):
-        input_elements = [i.describe_xml() for i in self.inputs]
-        output_elements = [i.describe_xml() for i in self.outputs]
-
-        doc = E.ProcessDescription(
-            OWS.Identifier(self.identifier),
-            OWS.Title(self.title)
-        )
-        doc.attrib['{http://www.opengis.net/wps/1.0.0}processVersion'] = self.version
-
-        if self.store_supported == 'true':
-            doc.attrib['storeSupported'] = self.store_supported
-
-        if self.status_supported == 'true':
-            doc.attrib['statusSupported'] = self.status_supported
-
-        if self.abstract:
-            doc.append(OWS.Abstract(self.abstract))
-
-        for m in self.metadata:
-            doc.append(OWS.Metadata(dict(m)))
-
-        for p in self.profile:
-            doc.append(WPS.Profile(p))
-
-        if input_elements:
-            doc.append(E.DataInputs(*input_elements))
-
-        doc.append(E.ProcessOutputs(*output_elements))
-
-        return doc
+        return {
+            'version': self.version,
+            'identifier': self.identifier,
+            'title': self.title,
+            'abstract': self.abstract,
+            'keywords': self.keywords,
+            'metadata': [m for m in self.metadata],
+            'inputs': [i.json for i in self.inputs],
+            'outputs': [o.json for o in self.outputs],
+            'store_supported': self.store_supported,
+            'status_supported': self.status_supported,
+            'profile': [p for p in self.profile],
+        }
 
     def execute(self, wps_request, uuid):
         self._set_uuid(uuid)
         self.async = False
-        wps_response = WPSResponse(self, wps_request, self.uuid)
+        response_cls = get_response("execute")
+        wps_response = response_cls(wps_request, process=self, uuid=self.uuid)
 
         LOGGER.debug('Check if status storage and updating are supported by this process')
         if wps_request.store_execute == 'true':
@@ -137,10 +109,10 @@ class Process(object):
                 if self.status_supported != 'true':
                     raise OperationNotSupported('Process does not support the updating of status')
 
-                wps_response.status = STATUS.STORE_AND_UPDATE_STATUS
+                wps_response.store_status_file = True
                 self.async = True
             else:
-                wps_response.status = STATUS.STORE_STATUS
+                wps_response.store_status_file = False
 
         LOGGER.debug('Check if updating of status is not required then no need to spawn a process')
 
@@ -149,10 +121,15 @@ class Process(object):
         return wps_response
 
     def _set_uuid(self, uuid):
-        """Set uuid and status ocation apth and url
+        """Set uuid and status location path and url
         """
 
         self.uuid = uuid
+        for inpt in self.inputs:
+            inpt.uuid = uuid
+
+        for outpt in self.outputs:
+            outpt.uuid = uuid
 
         file_path = config.get_config_value('server', 'outputpath')
 
@@ -162,7 +139,7 @@ class Process(object):
         self.status_url = os.path.join(file_url, str(self.uuid)) + '.xml'
 
     def _execute_process(self, async, wps_request, wps_response):
-        """Uses :module:`multiprocessing` module for sending process to
+        """Uses :module:`pywps.processing` module for sending process to
         background BUT first, check for maxprocesses configuration value
 
         :param async: run in asynchronous mode
@@ -178,53 +155,54 @@ class Process(object):
 
             # run immedietly
             if running < maxparallel or maxparallel == -1:
+                wps_response._update_status(WPS_STATUS.ACCEPTED, u"PyWPS Request accepted", 0)
                 self._run_async(wps_request, wps_response)
 
             # try to store for later usage
             else:
-                wps_response = self._store_process(stored,
-                                                   wps_request, wps_response)
+                maxprocesses = int(config.get_config_value('server', 'maxprocesses'))
+                if stored >= maxprocesses:
+                    raise ServerBusy('Maximum number of parallel running processes reached. Please try later.')
+                LOGGER.debug("Store process in job queue, uuid=%s", self.uuid)
+                dblog.store_process(self.uuid, wps_request)
+                wps_response._update_status(WPS_STATUS.ACCEPTED, u'PyWPS Process stored in job queue', 0)
 
         # not async
         else:
-            if running < maxparallel or maxparallel == -1:
-                wps_response = self._run_process(wps_request, wps_response)
-            else:
+            if running >= maxparallel and maxparallel != -1:
                 raise ServerBusy('Maximum number of parallel running processes reached. Please try later.')
+            wps_response._update_status(WPS_STATUS.ACCEPTED, u"PyWPS Request accepted", 0)
+            wps_response = self._run_process(wps_request, wps_response)
 
         return wps_response
 
+    # This function may not raise exception and must return a valid wps_response
+    # Failure must be reported as wps_response.status = WPS_STATUS.FAILED
     def _run_async(self, wps_request, wps_response):
-        import multiprocessing
-        process = multiprocessing.Process(
-            target=self._run_process,
-            args=(wps_request, wps_response)
-        )
+        import pywps.processing
+        process = pywps.processing.Process(
+            process=self,
+            wps_request=wps_request,
+            wps_response=wps_response)
         process.start()
 
-    def _store_process(self, stored, wps_request, wps_response):
-        """Try to store given requests
-        """
-
-        maxprocesses = int(config.get_config_value('server', 'maxprocesses'))
-
-        if stored < maxprocesses:
-            dblog.store_process(self.uuid, wps_request)
-        else:
-            raise ServerBusy('Maximum number of parallel running processes reached. Please try later.')
-
-        return wps_response
-
+    # This function may not raise exception and must return a valid wps_response
+    # Failure must be reported as wps_response.status = WPS_STATUS.FAILED
     def _run_process(self, wps_request, wps_response):
         try:
-            self._set_grass()
-            wps_response.update_status('PyWPS Process started', 0)
-            wps_response = self.handler(wps_request, wps_response)
-
-            # if (not wps_response.status_percentage) or (wps_response.status_percentage != 100):
-            LOGGER.debug('Updating process status to 100% if everything went correctly')
-            wps_response.update_status('PyWPS Process {} finished'.format(self.title),
-                                       100, STATUS.DONE_STATUS, clean=self.async)
+            self._set_grass(wps_request)
+            # if required set HOME to the current working directory.
+            if config.get_config_value('server', 'sethomedir') is True:
+                os.environ['HOME'] = self.workdir
+                LOGGER.info('Setting HOME to current working directory: %s', os.environ['HOME'])
+            LOGGER.debug('ProcessID=%s, HOME=%s', self.uuid, os.environ.get('HOME'))
+            wps_response._update_status(WPS_STATUS.STARTED, u'PyWPS Process started', 0)
+            self.handler(wps_request, wps_response)  # the user must update the wps_response.
+            # Ensure process termination
+            if wps_response.status != WPS_STATUS.SUCCEEDED and wps_response.status != WPS_STATUS.FAILED:
+                # if (not wps_response.status_percentage) or (wps_response.status_percentage != 100):
+                LOGGER.debug('Updating process status to 100% if everything went correctly')
+                wps_response._update_status(WPS_STATUS.SUCCEEDED, 'PyWPS Process {} finished'.format(self.title), 100)
         except Exception as e:
             traceback.print_exc()
             LOGGER.debug('Retrieving file and line number where exception occurred')
@@ -246,42 +224,61 @@ class Process(object):
             method_name = exc_tb.tb_frame.f_code.co_name
 
             # update the process status to display process failed
+
             msg = 'Process error: %s.%s Line %i %s' % (fname, method_name, exc_tb.tb_lineno, e)
             LOGGER.error(msg)
+            if config.get_config_value("logging", "level") != "DEBUG":
+                msg = 'Process failed, please check server error log'
+            wps_response._update_status(WPS_STATUS.FAILED, msg, 100)
 
-            if not wps_response:
-                raise NoApplicableCode('Response is empty. Make sure the _handler method is returning a valid object.')
-            else:
-                wps_response.update_status(msg, -1)
-
-        # tr
-        stored_request = dblog.get_first_stored()
-        if stored_request:
-            (uuid, request_json) = (stored_request.uuid, stored_request.request)
-            new_wps_request = WPSRequest()
-            new_wps_request.json = json.loads(request_json)
-            new_wps_response = WPSResponse(self, new_wps_request, uuid)
-            new_wps_response.status = STATUS.STORE_AND_UPDATE_STATUS
-            self._set_uuid(uuid)
-            self._run_async(new_wps_request, new_wps_response)
-            dblog.remove_stored(uuid)
+        finally:
+            # The run of the next pending request if finished here, weather or not it successfull
+            self.launch_next_process()
 
         return wps_response
 
+    def launch_next_process(self):
+        """Look at the queue of async process, if the queue is not empty launch the next pending request.
+        """
+        try:
+            LOGGER.debug("Checking for stored requests")
+
+            stored_request = dblog.get_first_stored()
+            if not stored_request:
+                LOGGER.debug("No stored request found")
+                return
+
+            (uuid, request_json) = (stored_request.uuid, stored_request.request)
+            if not PY2:
+                request_json = request_json.decode('utf-8')
+            LOGGER.debug("Launching the stored request %s", str(uuid))
+            new_wps_request = WPSRequest()
+            new_wps_request.json = json.loads(request_json)
+            process_identifier = new_wps_request.identifier
+            process = self.service.prepare_process_for_execution(process_identifier)
+            process._set_uuid(uuid)
+            process.async = True
+            new_wps_response = ExecuteResponse(new_wps_request, process=process, uuid=uuid)
+            process._run_async(new_wps_request, new_wps_response)
+            dblog.remove_stored(uuid)
+        except Exception as e:
+            LOGGER.error("Could not run stored process. %s", e)
+
     def clean(self):
         """Clean the process working dir and other temporary files
         """
-        LOGGER.info("Removing temporary working directory: %s" % self.workdir)
-        try:
-            if os.path.isdir(self.workdir):
-                shutil.rmtree(self.workdir)
-            if self._grass_mapset and os.path.isdir(self._grass_mapset):
-                LOGGER.info("Removing temporary GRASS GIS mapset: %s" % self._grass_mapset)
-                shutil.rmtree(self._grass_mapset)
-        except WindowsError as err:
-                LOGGER.error('Windows Error: %s', err)
-        except Exception as err:
+        if config.get_config_value('server', 'cleantempdir'):
+            LOGGER.info("Removing temporary working directory: %s" % self.workdir)
+            try:
+                if os.path.isdir(self.workdir):
+                    shutil.rmtree(self.workdir)
+                if self._grass_mapset and os.path.isdir(self._grass_mapset):
+                    LOGGER.info("Removing temporary GRASS GIS mapset: %s" % self._grass_mapset)
+                    shutil.rmtree(self._grass_mapset)
+            except Exception as err:
                 LOGGER.error('Unable to remove directory: %s', err)
+        else:
+            LOGGER.warning('Temporary working directory is not removed: %s' % self.workdir)
 
     def set_workdir(self, workdir):
         """Set working dir for all inputs and outputs
@@ -296,10 +293,11 @@ class Process(object):
         for outpt in self.outputs:
             outpt.workdir = workdir
 
-    def _set_grass(self):
+    def _set_grass(self, wps_request):
         """Handle given grass_location parameter of the constructor
 
-        location is either directory name or 'epsg:1234' form
+        location is either directory name, 'epsg:1234' form or a georeferenced
+        file
 
         in the first case, new temporary mapset within the location will be
         created
@@ -310,16 +308,14 @@ class Process(object):
         """
 
         if not PY2:
-            LOGGER.warning('Seems PyWPS is running in Python-3 ' +
+            LOGGER.warning('Seems PyWPS is running in Python-3 '
                            'environment, but GRASS GIS supports Python-2 only')
             return
 
         if self.grass_location:
 
             from grass.script import core as grass
-
-            dbase = ''
-            location = ''
+            from grass.script import setup as gsetup
 
             # HOME needs to be set - and that is usually not the case for httpd
             # server
@@ -332,15 +328,27 @@ class Process(object):
             gisrc.close()
             os.environ['GISRC'] = gisrc.name
 
-            # create new location from epsg code
-            if self.grass_location.lower().startswith('epsg:'):
+            new_loc_args = dict()
+
+            if self.grass_location.startswith('complexinput:'):
+                # create new location from a georeferenced file
+                ref_file_parameter = self.grass_location.split(':')[1]
+                ref_file = wps_request.inputs[ref_file_parameter][0].file
+                new_loc_args.update({'filename': ref_file})
+            elif self.grass_location.lower().startswith('epsg:'):
+                # create new location from epsg code
                 epsg = self.grass_location.lower().replace('epsg:', '')
+                new_loc_args.update({'epsg': epsg})
+
+            if new_loc_args:
                 dbase = self.workdir
-                os.environ['GISDBASE'] = self.workdir
                 location = 'pywps_location'
-                grass.run_command('g.gisenv', set="GISDBASE=%s" % dbase)
-                grass.run_command('g.proj', flags="t", location=location, epsg=epsg)
-                LOGGER.debug('GRASS location based on EPSG code created')
+                gsetup.init(self.workdir, dbase, location, 'PERMANENT')
+                grass.create_location(dbase=dbase,
+                                      location=location,
+                                      **new_loc_args)
+                LOGGER.debug('GRASS location based on {} created'.format(
+                    list(new_loc_args.keys())[0]))
 
             # create temporary mapset within existing location
             elif os.path.isdir(self.grass_location):
@@ -350,27 +358,22 @@ class Process(object):
                 grass.run_command('g.gisenv', set="GISDBASE=%s" % dbase)
 
             else:
-                raise NoApplicableCode('Location does exists or does not seem ' +
+                raise NoApplicableCode('Location does exists or does not seem '
                                        'to be in "EPSG:XXXX" form nor is it existing directory: %s' % location)
 
             # copy projection files from PERMAMENT mapset to temporary mapset
-            mapset_name = tempfile.mkdtemp(prefix='pywps_', dir=os.path.join(dbase, location))
-            shutil.copy(os.path.join(dbase, location, 'PERMANENT',
-                        'DEFAULT_WIND'), os.path.join(mapset_name, 'WIND'))
-            shutil.copy(os.path.join(dbase, location, 'PERMANENT',
-                        'PROJ_EPSG'), os.path.join(mapset_name, 'PROJ_EPSG'))
-            shutil.copy(os.path.join(dbase, location, 'PERMANENT',
-                        'PROJ_INFO'), os.path.join(mapset_name, 'PROJ_INFO'))
-            shutil.copy(os.path.join(dbase, location, 'PERMANENT',
-                        'PROJ_UNITS'), os.path.join(mapset_name, 'PROJ_UNITS'))
+            mapset_name = 'pywps_mapset'
+            grass.run_command('g.mapset',
+                              mapset=mapset_name,
+                              flags='c',
+                              dbase=dbase,
+                              location=location)
 
             # set _grass_mapset attribute - will be deleted once handler ends
             self._grass_mapset = mapset_name
 
             # final initialization
             LOGGER.debug('GRASS Mapset set to %s' % mapset_name)
-            grass.run_command('g.gisenv', set="LOCATION_NAME=%s" % location)
-            grass.run_command('g.gisenv', set="MAPSET=%s" % os.path.basename(mapset_name))
 
             LOGGER.debug('GRASS environment initialised')
             LOGGER.debug('GISRC {}, GISBASE {}, GISDBASE {}, LOCATION {}, MAPSET {}'.format(
