@@ -1,26 +1,25 @@
 ##################################################################
-# Copyright 2016 OSGeo Foundation,                               #
-# represented by PyWPS Project Steering Committee,               #
+# Copyright 2018 Open Source Geospatial Foundation and others    #
 # licensed under MIT, Please consult LICENSE.txt for details     #
 ##################################################################
 
-
 import logging
 import lxml
 import lxml.etree
 from werkzeug.exceptions import MethodNotAllowed
+from pywps import get_ElementMakerForVersion
 import base64
-from pywps import WPS
+import datetime
 from pywps._compat import text_type, PY2
-from pywps.app.basic import xpath_ns
+from pywps.app.basic import get_xpath_ns
 from pywps.inout.basic import LiteralInput, ComplexInput, BBoxInput
 from pywps.exceptions import NoApplicableCode, OperationNotSupported, MissingParameterValue, VersionNegotiationFailed, \
     InvalidParameterValue, FileSizeExceeded
 from pywps import configuration
 from pywps.validator.mode import MODE
 from pywps.inout.literaltypes import AnyValue, NoValue, ValuesReference, AllowedValue
-
 from pywps.inout.formats import Format
+from pywps import get_version_from_ns
 
 import json
 
@@ -35,13 +34,17 @@ class WPSRequest(object):
         self.operation = None
         self.version = None
         self.language = None
+        self.identifier = None
         self.identifiers = None
         self.store_execute = None
         self.status = None
         self.lineage = None
-        self.inputs = None
-        self.outputs = None
+        self.inputs = {}
+        self.outputs = {}
         self.raw = None
+        self.WPS = None
+        self.OWS = None
+        self.xpath_ns = None
 
         if self.http_request:
             request_parser = self._get_request_parser_method(http_request.method)
@@ -82,7 +85,7 @@ class WPSRequest(object):
         maxsize = configuration.get_size_mb(maxsize) * 1024 * 1024
         if self.http_request.content_length > maxsize:
             raise FileSizeExceeded('File size for input exceeded.'
-                                   ' Maximum request size allowed: %i megabytes' % maxsize / 1024 / 1024)
+                                   ' Maximum request size allowed: %i megabytes' % (maxsize / 1024 / 1024))
 
         try:
             doc = lxml.etree.fromstring(self.http_request.get_data())
@@ -93,6 +96,8 @@ class WPSRequest(object):
                 raise NoApplicableCode(e.msg)
 
         operation = doc.tag
+        version = get_version_from_ns(doc.nsmap[doc.prefix])
+        self.set_version(version)
         request_parser = self._post_request_parser(operation)
         request_parser(doc)
 
@@ -138,6 +143,9 @@ class WPSRequest(object):
                 http_request, 'lineage', 'false')
             wpsrequest.inputs = get_data_from_kvp(
                 _get_get_param(http_request, 'DataInputs'), 'DataInputs')
+            if self.inputs is None:
+                self.inputs = {}
+
             wpsrequest.outputs = {}
 
             # take responseDocument preferably
@@ -180,7 +188,7 @@ class WPSRequest(object):
         def parse_post_getcapabilities(doc):
             """Parse POST GetCapabilities request
             """
-            acceptedversions = xpath_ns(
+            acceptedversions = self.xpath_ns(
                 doc, '/wps:GetCapabilities/ows:AcceptVersions/ows:Version')
             acceptedversions = ','.join(
                 map(lambda v: v.text, acceptedversions))
@@ -198,7 +206,7 @@ class WPSRequest(object):
 
             wpsrequest.operation = 'describeprocess'
             wpsrequest.identifiers = [identifier_el.text for identifier_el in
-                                      xpath_ns(doc, './ows:Identifier')]
+                                      self.xpath_ns(doc, './ows:Identifier')]
 
         def parse_post_execute(doc):
             """Parse POST Execute request
@@ -212,7 +220,7 @@ class WPSRequest(object):
 
             wpsrequest.operation = 'execute'
 
-            identifier = xpath_ns(doc, './ows:Identifier')
+            identifier = self.xpath_ns(doc, './ows:Identifier')
 
             if not identifier:
                 raise MissingParameterValue(
@@ -225,13 +233,13 @@ class WPSRequest(object):
             wpsrequest.inputs = get_inputs_from_xml(doc)
             wpsrequest.outputs = get_output_from_xml(doc)
             wpsrequest.raw = False
-            if xpath_ns(doc, '/wps:Execute/wps:ResponseForm/wps:RawDataOutput'):
+            if self.xpath_ns(doc, '/wps:Execute/wps:ResponseForm/wps:RawDataOutput'):
                 wpsrequest.raw = True
                 # executeResponse XML will not be stored
                 wpsrequest.store_execute = 'false'
 
             # check if response document tag has been set then retrieve
-            response_document = xpath_ns(
+            response_document = self.xpath_ns(
                 doc, './wps:ResponseForm/wps:ResponseDocument')
             if len(response_document) > 0:
                 wpsrequest.lineage = response_document[
@@ -241,19 +249,24 @@ class WPSRequest(object):
                 wpsrequest.status = response_document[
                     0].attrib.get('status', 'false')
 
-        if tagname == WPS.GetCapabilities().tag:
+        if tagname == self.WPS.GetCapabilities().tag:
             self.operation = 'getcapabilities'
             return parse_post_getcapabilities
-        elif tagname == WPS.DescribeProcess().tag:
+        elif tagname == self.WPS.DescribeProcess().tag:
             self.operation = 'describeprocess'
             return parse_post_describeprocess
-        elif tagname == WPS.Execute().tag:
+        elif tagname == self.WPS.Execute().tag:
             self.operation = 'execute'
             return parse_post_execute
         else:
             raise InvalidParameterValue(
                 'Unknown request %r' % tagname, 'request')
 
+    def set_version(self, version):
+        self.version = version
+        self.xpath_ns = get_xpath_ns(version)
+        self.WPS, self.OWS = get_ElementMakerForVersion(self.version)
+
     def check_accepted_versions(self, acceptedversions):
         """
         :param acceptedversions: string
@@ -285,7 +298,7 @@ class WPSRequest(object):
             raise VersionNegotiationFailed(
                 'The requested version "%s" is not supported by this server' % version, 'version')
         else:
-            self.version = version
+            self.set_version(version)
 
     def check_and_set_language(self, language):
         """set this.language
@@ -303,11 +316,19 @@ class WPSRequest(object):
     def json(self):
         """Return JSON encoded representation of the request
         """
+        class ExtendedJSONEncoder(json.JSONEncoder):
+            def default(self, obj):
+                if isinstance(obj, datetime.date) or isinstance(obj, datetime.time):
+                    encoded_object = obj.isoformat()
+                else:
+                    encoded_object = json.JSONEncoder.default(self, obj)
+                return encoded_object
 
         obj = {
             'operation': self.operation,
             'version': self.version,
             'language': self.language,
+            'identifier': self.identifier,
             'identifiers': self.identifiers,
             'store_execute': self.store_execute,
             'status': self.status,
@@ -317,7 +338,7 @@ class WPSRequest(object):
             'raw': self.raw
         }
 
-        return json.dumps(obj, allow_nan=False)
+        return json.dumps(obj, allow_nan=False, cls=ExtendedJSONEncoder)
 
     @json.setter
     def json(self, value):
@@ -329,6 +350,7 @@ class WPSRequest(object):
         self.operation = value['operation']
         self.version = value['version']
         self.language = value['language']
+        self.identifier = value['identifier']
         self.identifiers = value['identifiers']
         self.store_execute = value['store_execute']
         self.status = value['status']
@@ -420,6 +442,8 @@ class WPSRequest(object):
 
 def get_inputs_from_xml(doc):
     the_inputs = {}
+    version = get_version_from_ns(doc.nsmap[doc.prefix])
+    xpath_ns = get_xpath_ns(version)
     for input_el in xpath_ns(doc, '/wps:Execute/wps:DataInputs/wps:Input'):
         [identifier_el] = xpath_ns(input_el, './ows:Identifier')
         identifier = identifier_el.text
@@ -497,11 +521,18 @@ def get_inputs_from_xml(doc):
 def get_output_from_xml(doc):
     the_output = {}
 
+    version = get_version_from_ns(doc.nsmap[doc.prefix])
+    xpath_ns = get_xpath_ns(version)
+
     if xpath_ns(doc, '/wps:Execute/wps:ResponseForm/wps:ResponseDocument'):
         for output_el in xpath_ns(doc, '/wps:Execute/wps:ResponseForm/wps:ResponseDocument/wps:Output'):
             [identifier_el] = xpath_ns(output_el, './ows:Identifier')
             outpt = {}
             outpt[identifier_el.text] = ''
+            outpt['mimetype'] = output_el.attrib.get('mimeType', '')
+            outpt['encoding'] = output_el.attrib.get('encoding', '')
+            outpt['schema'] = output_el.attrib.get('schema', '')
+            outpt['uom'] = output_el.attrib.get('uom', '')
             outpt['asReference'] = output_el.attrib.get('asReference', 'false')
             the_output[identifier_el.text] = outpt
 
@@ -542,7 +573,7 @@ def get_data_from_kvp(data, part=None):
 
             # Get the attributes of the data
             for attr in fields[1:]:
-                (attribute, attr_val) = attr.split('=')
+                (attribute, attr_val) = attr.split('=', 1)
                 if attribute == 'xlink:href':
                     io['href'] = attr_val
                 else:
@@ -566,7 +597,7 @@ def get_data_from_kvp(data, part=None):
 def _check_version(version):
     """ check given version
     """
-    if version != '1.0.0':
+    if version not in ['1.0.0', '2.0.0']:
         return False
     else:
         return True
@@ -616,7 +647,7 @@ def _get_rawvalue_value(data, encoding=None):
         elif encoding == 'base64':
             return base64.b64decode(data)
         return base64.b64decode(data)
-    except:
+    except Exception:
         return data
 
 
