@@ -1,13 +1,10 @@
 ##################################################################
-# Copyright 2016 OSGeo Foundation,                               #
-# represented by PyWPS Project Steering Committee,               #
+# Copyright 2018 Open Source Geospatial Foundation and others    #
 # licensed under MIT, Please consult LICENSE.txt for details     #
 ##################################################################
 
-
 from pywps.app.Process import Process  # noqa: F401
 from pywps.app.Service import Service  # noqa: F401
-from pywps.app.WPSResponse import WPSResponse  # noqa: F401
 from pywps.app.WPSRequest import WPSRequest  # noqa: F401
 from pywps.app.WPSRequest import get_inputs_from_xml  # noqa: F401
 from pywps.app.WPSRequest import get_output_from_xml  # noqa: F401
