@@ -1,6 +1,5 @@
 ##################################################################
-# Copyright 2016 OSGeo Foundation,                               #
-# represented by PyWPS Project Steering Committee,               #
+# Copyright 2018 Open Source Geospatial Foundation and others    #
 # licensed under MIT, Please consult LICENSE.txt for details     #
 ##################################################################
 
@@ -22,6 +21,7 @@ else:
 
 __author__ = "Calin Ciociu"
 
+RAW_OPTIONS = [('logging', 'format'), ]
 
 CONFIG = None
 LOGGER = logging.getLogger("PYWPS")
@@ -44,7 +44,8 @@ def get_config_value(section, option):
 
     if CONFIG.has_section(section):
         if CONFIG.has_option(section, option):
-            value = CONFIG.get(section, option)
+            raw = (section, option) in RAW_OPTIONS
+            value = CONFIG.get(section, option, raw=raw)
 
             # Convert Boolean string to real Boolean values
             if value.lower() == "false":
@@ -67,9 +68,9 @@ def load_configuration(cfgfiles=None):
 
     LOGGER.info('loading configuration')
     if PY2:
-        CONFIG = ConfigParser.SafeConfigParser()
+        CONFIG = ConfigParser.SafeConfigParser(os.environ)
     else:
-        CONFIG = configparser.ConfigParser()
+        CONFIG = configparser.ConfigParser(os.environ)
 
     LOGGER.debug('setting default values')
     CONFIG.add_section('server')
@@ -82,16 +83,29 @@ def load_configuration(cfgfiles=None):
     CONFIG.set('server', 'temp_path', tempfile.gettempdir())
     CONFIG.set('server', 'processes_path', '')
     outputpath = tempfile.gettempdir()
-    CONFIG.set('server', 'outputurl', 'file:///%s' % outputpath)
+    CONFIG.set('server', 'outputurl', 'file://%s' % outputpath)
     CONFIG.set('server', 'outputpath', outputpath)
+    # list of allowed input paths (file url input) seperated by ':'
+    CONFIG.set('server', 'allowedinputpaths', '')
     CONFIG.set('server', 'workdir', tempfile.gettempdir())
     CONFIG.set('server', 'parallelprocesses', '2')
+    # If this flag is enabled it will set the HOME environment
+    # for each process to its current workdir (a temp folder).
+    CONFIG.set('server', 'sethomedir', 'false')
+    # If this flag is enabled PyWPS will remove the process temporary workdir
+    # after process has finished.
+    CONFIG.set('server', 'cleantempdir', 'true')
+
+    CONFIG.add_section('processing')
+    CONFIG.set('processing', 'mode', 'default')
+    CONFIG.set('processing', 'path', os.path.dirname(os.path.realpath(sys.argv[0])))
 
     CONFIG.add_section('logging')
     CONFIG.set('logging', 'file', '')
     CONFIG.set('logging', 'level', 'DEBUG')
     CONFIG.set('logging', 'database', 'sqlite:///:memory:')
     CONFIG.set('logging', 'prefix', 'pywps_')
+    CONFIG.set('logging', 'format', '%(asctime)s] [%(levelname)s] file=%(pathname)s line=%(lineno)s module=%(module)s function=%(funcName)s %(message)s')  # noqa
 
     CONFIG.add_section('metadata:main')
     CONFIG.set('metadata:main', 'identification_title', 'PyWPS Processing Service')
@@ -204,7 +218,7 @@ def _get_default_config_files_location():
 
 
 def get_size_mb(mbsize):
-    """Get real size of given obeject
+    """Get real size of given object in Mb.
 
     """
 
