@@ -1,6 +1,5 @@
 ##################################################################
-# Copyright 2016 OSGeo Foundation,                               #
-# represented by PyWPS Project Steering Committee,               #
+# Copyright 2018 Open Source Geospatial Foundation and others    #
 # licensed under MIT, Please consult LICENSE.txt for details     #
 ##################################################################
 
@@ -15,7 +14,7 @@ from pywps.exceptions import InvalidParameterValue
 from pywps.validator.allowed_value import RANGECLOSURETYPE
 from pywps.validator.allowed_value import ALLOWEDVALUETYPE
 from pywps._compat import PY2
-from pywps import OWS, NAMESPACES
+from pywps import get_ElementMakerForVersion
 
 import logging
 LOGGER = logging.getLogger('PYWPS')
@@ -90,21 +89,6 @@ class AllowedValue(AnyValue):
         self.spacing = spacing
         self.range_closure = range_closure
 
-    def describe_xml(self):
-        """Return back Element for DescribeProcess response
-        """
-        doc = None
-        if self.allowed_type == ALLOWEDVALUETYPE.VALUE:
-            doc = OWS.Value(str(self.value))
-        else:
-            doc = OWS.Range()
-            doc.set('{%s}rangeClosure' % NAMESPACES['ows'], self.range_closure)
-            doc.append(OWS.MinimumValue(str(self.minval)))
-            doc.append(OWS.MaximumValue(str(self.maxval)))
-            if self.spacing:
-                doc.append(OWS.Spacing(str(self.spacing)))
-        return doc
-
     @property
     def json(self):
         value = self.value
@@ -154,7 +138,7 @@ def get_converter(convertor):
                 convert = convert_positiveInteger
             else:
                 raise InvalidParameterValue(
-                    "Invalid data_type value of LiteralInput " +
+                    "Invalid data_type value of LiteralInput "
                     "set to '{}'".format(data_type))
         try:
             return convert(data)
@@ -199,7 +183,7 @@ def convert_boolean(inpt):
                 val = False
             else:
                 val = True
-        except:
+        except Exception:
             val = True
     return val
 
