@@ -1,20 +1,22 @@
 ##################################################################
-# Copyright 2016 OSGeo Foundation,                               #
-# represented by PyWPS Project Steering Committee,               #
+# Copyright 2018 Open Source Geospatial Foundation and others    #
 # licensed under MIT, Please consult LICENSE.txt for details     #
 ##################################################################
 
 import unittest
 import lxml.etree
 import json
+import tempfile
+import os.path
 from pywps import Service, Process, LiteralOutput, LiteralInput,\
-    BoundingBoxOutput, BoundingBoxInput, Format, ComplexInput, ComplexOutput
+    BoundingBoxOutput, BoundingBoxInput, Format, ComplexInput, ComplexOutput, FORMATS
 from pywps.validator.base import emptyvalidator
 from pywps.validator.complexvalidator import validategml
+from pywps.validator.mode import MODE
 from pywps.exceptions import InvalidParameterValue
 from pywps import get_inputs_from_xml, get_output_from_xml
-from pywps import E, WPS, OWS
-from pywps.app.basic import xpath_ns
+from pywps import E, get_ElementMakerForVersion
+from pywps.app.basic import get_xpath_ns
 from pywps._compat import text_type
 from pywps.tests import client_for, assert_response_success
 
@@ -23,6 +25,21 @@ from pywps._compat import StringIO
 if PY2:
     from owslib.ows import BoundingBox
 
+try:
+    import netCDF4
+except ImportError:
+    WITH_NC4 = False
+else:
+    WITH_NC4 = True
+
+DATA_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'data')
+
+VERSION = "1.0.0"
+
+WPS, OWS = get_ElementMakerForVersion(VERSION)
+
+xpath_ns = get_xpath_ns(VERSION)
+
 
 def create_ultimate_question():
     def handler(request, response):
@@ -48,10 +65,11 @@ def create_greeter():
                    inputs=[LiteralInput('name', 'Input name', data_type='string')],
                    outputs=[LiteralOutput('message', 'Output message', data_type='string')])
 
+
 def create_bbox_process():
     def bbox_process(request, response):
         coords = request.inputs['mybbox'][0].data
-        assert type(coords) == type([])
+        assert isinstance(coords, list)
         assert len(coords) == 4
         assert coords[0] == '15'
         response.outputs['outbbox'].data = coords
@@ -63,28 +81,87 @@ def create_bbox_process():
                    inputs=[BoundingBoxInput('mybbox', 'Input name', ["EPSG:4326"])],
                    outputs=[BoundingBoxOutput('outbbox', 'Output message', ["EPSG:4326"])])
 
+
 def create_complex_proces():
     def complex_proces(request, response):
         response.outputs['complex'].data = request.inputs['complex'][0].data
         return response
 
-    frmt = Format(mime_type='application/gml') # this is unknown mimetype
+    frmt = Format(mime_type='application/gml', extension=".gml")  # this is unknown mimetype
 
     return Process(handler=complex_proces,
-            identifier='my_complex_process',
-            title='Complex process',
-            inputs=[
-                ComplexInput(
-                    'complex',
-                    'Complex input',
-                    supported_formats=[frmt])
-            ],
-            outputs=[
-                ComplexOutput(
-                    'complex',
-                    'Complex output',
-                    supported_formats=[frmt])
-             ])
+                   identifier='my_complex_process',
+                   title='Complex process',
+                   inputs=[
+                       ComplexInput(
+                           'complex',
+                           'Complex input',
+                           default="DEFAULT COMPLEX DATA",
+                           supported_formats=[frmt])
+                   ],
+                   outputs=[
+                       ComplexOutput(
+                           'complex',
+                           'Complex output',
+                           supported_formats=[frmt])
+                   ])
+
+
+def create_complex_nc_process():
+    def complex_proces(request, response):
+        from pywps.dependencies import netCDF4 as nc
+        url = request.inputs['dods'][0].url
+        with nc.Dataset(url) as D:
+            response.outputs['conventions'].data = D.Conventions
+
+        response.outputs['outdods'].url = url
+        response.outputs['ncraw'].file = os.path.join(DATA_DIR, 'netcdf', 'time.nc')
+        response.outputs['ncraw'].data_format = FORMATS.NETCDF
+        return response
+
+    return Process(handler=complex_proces,
+                   identifier='my_opendap_process',
+                   title='Opendap process',
+                   inputs=[
+                       ComplexInput(
+                           'dods',
+                           'Opendap input',
+                           supported_formats=[Format('DODS'), Format('NETCDF')],
+                           #   mode=MODE.STRICT
+                       )
+                   ],
+                   outputs=[
+                       LiteralOutput(
+                           'conventions',
+                           'NetCDF convention',
+                       ),
+                       ComplexOutput('outdods', 'Opendap output',
+                                     supported_formats=[FORMATS.DODS, ],
+                                     as_reference=True),
+                       ComplexOutput('ncraw', 'NetCDF raw data output',
+                                     supported_formats=[FORMATS.NETCDF, ],
+                                     as_reference=False)
+                   ])
+
+
+def create_mimetype_process():
+    def _handler(request, response):
+        response.outputs['mimetype'].data = response.outputs['mimetype'].data_format.mime_type
+        return response
+
+    frmt_txt = Format(mime_type='text/plain')
+    frmt_txt2 = Format(mime_type='text/plain+test')
+
+    return Process(handler=_handler,
+                   identifier='get_mimetype_process',
+                   title='Get mimeType process',
+                   inputs=[],
+                   outputs=[
+                       ComplexOutput(
+                           'mimetype',
+                           'mimetype of requested output',
+                           supported_formats=[frmt_txt, frmt_txt2])
+                   ])
 
 
 def get_output(doc):
@@ -100,6 +177,60 @@ def get_output(doc):
 class ExecuteTest(unittest.TestCase):
     """Test for Exeucte request KVP request"""
 
+    def test_dods(self):
+        if not WITH_NC4:
+            self.skipTest('netCDF4 not installed')
+        my_process = create_complex_nc_process()
+        service = Service(processes=[my_process])
+
+        href = "http://test.opendap.org:80/opendap/netcdf/examples/sresa1b_ncar_ccsm3_0_run1_200001.nc"
+
+        """ # Is this how the request should be written ?
+        request_doc = WPS.Execute(
+            OWS.Identifier('my_opendap_process'),
+            WPS.DataInputs(
+                WPS.Input(
+                    OWS.Identifier('dods'),
+                    WPS.Reference(
+                        WPS.Body('request body'),
+                        {'{http://www.w3.org/1999/xlink}href': href},
+                        method='POST'
+                    )
+                    #WPS.Data(WPS.ComplexData(href=href, mime_type='application/x-ogc-dods'))
+                    # This form is not supported yet. Should it be ?
+                )
+            ),
+            version='1.0.0'
+        )
+        resp = client.post_xml(doc=request_doc)
+        assert_response_success(resp)
+        """
+
+        class FakeRequest():
+            identifier = 'my_opendap_process'
+            service = 'wps'
+            operation = 'execute'
+            version = '1.0.0'
+            raw = True,
+            inputs = {'dods': [{
+                'identifier': 'dods',
+                'href': href,
+            }]}
+            store_execute = False
+            lineage = False
+            outputs = ['conventions']
+
+        request = FakeRequest()
+
+        resp = service.execute('my_opendap_process', request, 'fakeuuid')
+        self.assertEqual(resp.outputs['conventions'].data, u'CF-1.0')
+        self.assertEqual(resp.outputs['outdods'].url, href)
+        self.assertTrue(resp.outputs['outdods'].as_reference)
+        self.assertFalse(resp.outputs['ncraw'].as_reference)
+        with open(os.path.join(DATA_DIR, 'netcdf', 'time.nc'), 'rb') as f:
+            data = f.read()
+        self.assertEqual(resp.outputs['ncraw'].data, data)
+
     def test_input_parser(self):
         """Test input parsing
         """
@@ -110,14 +241,15 @@ class ExecuteTest(unittest.TestCase):
 
         class FakeRequest():
             identifier = 'complex_process'
-            service='wps'
-            version='1.0.0'
+            service = 'wps'
+            operation = 'execute'
+            version = '1.0.0'
             inputs = {'complex': [{
-                    'identifier': 'complex',
-                    'mimeType': 'text/gml',
-                    'data': 'the data'
-                }]}
-        request = FakeRequest();
+                'identifier': 'complex',
+                'mimeType': 'text/gml',
+                'data': 'the data'
+            }]}
+        request = FakeRequest()
 
         try:
             service.execute('my_complex_process', request, 'fakeuuid')
@@ -135,7 +267,7 @@ class ExecuteTest(unittest.TestCase):
         request.inputs['complex'][0]['mimeType'] = 'application/xml+gml'
         try:
             parsed_inputs = service.create_complex_inputs(my_process.inputs[0],
-                                                      request.inputs['complex'])
+                                                          request.inputs['complex'])
         except InvalidParameterValue as e:
             self.assertEqual(e.locator, 'mimeType')
 
@@ -150,10 +282,67 @@ class ExecuteTest(unittest.TestCase):
         my_process.inputs[0].supported_formats = [frmt]
         my_process.inputs[0].data_format = Format(mime_type='application/xml+gml')
         parsed_inputs = service.create_complex_inputs(my_process.inputs[0],
-                                              request.inputs['complex'])
+                                                      request.inputs['complex'])
 
         self.assertEqual(parsed_inputs[0].data_format.validate, validategml)
 
+    def test_input_default(self):
+        """Test input parsing
+        """
+        my_process = create_complex_proces()
+        service = Service(processes=[my_process])
+        self.assertEqual(len(service.processes.keys()), 1)
+        self.assertTrue(service.processes['my_complex_process'])
+
+        class FakeRequest():
+            identifier = 'complex_process'
+            service = 'wps'
+            operation = 'execute'
+            version = '1.0.0'
+            inputs = {}
+            raw = False
+            outputs = {}
+            store_execute = False
+            lineage = False
+
+        request = FakeRequest()
+        response = service.execute('my_complex_process', request, 'fakeuuid')
+        self.assertEqual(response.outputs['complex'].data, 'DEFAULT COMPLEX DATA')
+
+    def test_output_mimetype(self):
+        """Test input parsing
+        """
+        my_process = create_mimetype_process()
+        service = Service(processes=[my_process])
+        self.assertEqual(len(service.processes.keys()), 1)
+        self.assertTrue(service.processes['get_mimetype_process'])
+
+        class FakeRequest():
+            def __init__(self, mimetype):
+                self.outputs = {'mimetype': {
+                    'identifier': 'mimetype',
+                    'mimetype': mimetype,
+                    'data': 'the data'
+                }}
+
+            identifier = 'get_mimetype_process'
+            service = 'wps'
+            operation = 'execute'
+            version = '1.0.0'
+            inputs = {}
+            raw = False
+            store_execute = False
+            lineage = False
+
+        # valid mimetype
+        request = FakeRequest('text/plain+test')
+        response = service.execute('get_mimetype_process', request, 'fakeuuid')
+        self.assertEqual(response.outputs['mimetype'].data, 'text/plain+test')
+
+        # non valid mimetype
+        request = FakeRequest('text/xml')
+        with self.assertRaises(InvalidParameterValue):
+            response = service.execute('get_mimetype_process', request, 'fakeuuid')
 
     def test_missing_process_error(self):
         client = client_for(Service(processes=[create_ultimate_question()]))
@@ -173,6 +362,7 @@ class ExecuteTest(unittest.TestCase):
             OWS.Identifier('ultimate_question'),
             version='1.0.0'
         )
+
         resp = client.post_xml(doc=request_doc)
         assert_response_success(resp)
         assert get_output(resp.xml) == {'outvalue': '42'}
@@ -194,8 +384,7 @@ class ExecuteTest(unittest.TestCase):
         assert get_output(resp.xml) == {'message': "Hello foo!"}
 
     def test_bbox(self):
-        if not PY2:
-            self.skipTest('OWSlib not python 3 compatible')
+        self.skipTest('OWSlib not python 3 compatible')
         client = client_for(Service(processes=[create_bbox_process()]))
         request_doc = WPS.Execute(
             OWS.Identifier('my_bbox_process'),
@@ -205,7 +394,7 @@ class ExecuteTest(unittest.TestCase):
                     WPS.Data(WPS.BoundingBoxData(
                         OWS.LowerCorner('15 50'),
                         OWS.UpperCorner('16 51'),
-                        ))
+                    ))
                 )
             ),
             version='1.0.0'
@@ -214,11 +403,35 @@ class ExecuteTest(unittest.TestCase):
         assert_response_success(resp)
 
         [output] = xpath_ns(resp.xml, '/wps:ExecuteResponse'
-                                   '/wps:ProcessOutputs/Output')
-        self.assertEqual('outbbox', xpath_ns(output,
+                                      '/wps:ProcessOutputs/wps:Output')
+        self.assertEqual('outbbox', xpath_ns(
+            output,
             './ows:Identifier')[0].text)
-        self.assertEqual('15 50', xpath_ns(output,
-            './ows:BoundingBox/ows:LowerCorner')[0].text)
+
+        self.assertEqual('15 50', xpath_ns(
+            output,
+            './wps:Data/ows:BoundingBox/ows:LowerCorner')[0].text)
+
+        self.assertEqual('16 50', xpath_ns(
+            output,
+            './wps:Data/ows:BoundingBox/ows:LowerCorner')[0].text)
+
+    def test_output_response_dataType(self):
+        client = client_for(Service(processes=[create_greeter()]))
+        request_doc = WPS.Execute(
+            OWS.Identifier('greeter'),
+            WPS.DataInputs(
+                WPS.Input(
+                    OWS.Identifier('name'),
+                    WPS.Data(WPS.LiteralData('foo'))
+                )
+            ),
+            version='1.0.0'
+        )
+        resp = client.post_xml(doc=request_doc)
+        el = next(resp.xml.iter('{http://www.opengis.net/wps/1.0.0}LiteralData'))
+        assert el.attrib['dataType'] == 'string'
+
 
 class ExecuteXmlParserTest(unittest.TestCase):
     """Tests for Execute request XML Parser
@@ -238,7 +451,7 @@ class ExecuteXmlParserTest(unittest.TestCase):
                 WPS.Input(
                     OWS.Identifier('name'),
                     WPS.Data(WPS.LiteralData('bar')))
-                ))
+            ))
         rv = get_inputs_from_xml(request_doc)
         self.assertTrue('name' in rv)
         self.assertEqual(len(rv['name']), 2)
@@ -299,14 +512,13 @@ class ExecuteXmlParserTest(unittest.TestCase):
                     OWS.Identifier('json'),
                     WPS.Data(
                         WPS.ComplexData(the_data,
-                            encoding='base64',
-                            mimeType='application/json')))))
+                                        encoding='base64',
+                                        mimeType='application/json')))))
         rv = get_inputs_from_xml(request_doc)
         self.assertEqual(rv['json'][0]['mimeType'], 'application/json')
         json_data = json.loads(rv['json'][0]['data'].decode())
         self.assertEqual(json_data['plot']['Version'], '0.1')
 
-
     def test_bbox_input(self):
         if not PY2:
             self.skipTest('OWSlib not python 3 compatible')
@@ -354,7 +566,7 @@ class ExecuteXmlParserTest(unittest.TestCase):
                     OWS.Identifier('name'),
                     WPS.Reference(
                         WPS.BodyReference(
-                        {'{http://www.w3.org/1999/xlink}href': 'http://foo/bar/reference'}),
+                            {'{http://www.w3.org/1999/xlink}href': 'http://foo/bar/reference'}),
                         {'{http://www.w3.org/1999/xlink}href': 'http://foo/bar/service'},
                         method='POST'
                     )
@@ -365,6 +577,36 @@ class ExecuteXmlParserTest(unittest.TestCase):
         self.assertEqual(rv['name'][0]['href'], 'http://foo/bar/service')
         self.assertEqual(rv['name'][0]['bodyreference'], 'http://foo/bar/reference')
 
+    def test_build_input_file_name(self):
+        from pywps.inout.basic import ComplexInput
+
+        h = ComplexInput('ci')
+        h.workdir = workdir = tempfile.mkdtemp()
+
+        self.assertEqual(
+            h._build_file_name('http://path/to/test.txt'),
+            os.path.join(workdir, 'test.txt'))
+        self.assertEqual(
+            h._build_file_name('http://path/to/test'),
+            os.path.join(workdir, 'test'))
+        self.assertEqual(
+            h._build_file_name('file://path/to/.config'),
+            os.path.join(workdir, '.config'))
+        self.assertEqual(
+            h._build_file_name('https://path/to/test.txt?token=abc&expires_at=1234567'),
+            os.path.join(workdir, 'test.txt'))
+
+        h.supported_formats = [FORMATS.TEXT, ]
+        h.data_format = FORMATS.TEXT
+        self.assertEqual(
+            h._build_file_name('http://path/to/test'),
+            os.path.join(workdir, 'test.txt'))
+
+        open(os.path.join(workdir, 'duplicate.html'), 'a').close()
+        inpt_filename = h._build_file_name('http://path/to/duplicate.html')
+        self.assertTrue(inpt_filename.startswith(os.path.join(workdir, 'duplicate_')))
+        self.assertTrue(inpt_filename.endswith('.html'))
+
 
 def load_tests(loader=None, tests=None, pattern=None):
     if not loader:
